/*
 *  linux/amiga/config.c
 *
 *  Copyright (C) 1993 Hamish Macdonald
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file README.legal in the main directory of this archive
 * for more details.
 */

/*
 * Miscellaneous Amiga stuff
 */

#include <stdarg.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/interrupt.h>
#include <linux/mktime.h>
#include <linux/bootinfo.h>

#include <linux/amigahw.h>
#include <linux/amigaints.h>

#include <asm/system.h>

#if 0
caddr_t CustomBase;
caddr_t CiaABase;
caddr_t CiaBBase;
caddr_t ChipRamBase;
#endif

void amiga_sched_init (isrfunc timer_routine)
{
    /* number of ticks depends on NTSC or PAL CIA EClock */
    unsigned short ticks =
	boot_info.bi_amiga.eclock == NTSC_ECLOCK ?
	    NTSC_JIFFY_TICKS : PAL_JIFFY_TICKS;

    /* CIA interrupts when counter underflows, so adjust ticks by 1 */
    ticks -= 1;

    ciab.cra &= 0xC0;	 /* turn off timer A, continous mode, from Eclk */
    ciab.talo = ticks % 256;
    ciab.tahi = ticks / 256;

    /* install interrupt service routine for CIAB Timer A */
    add_isr (IRQ_AMIGA_CIAB_TA, timer_routine, 0, NULL);
    /* start timer */
    ciab.cra |= 0x01;
}

#define TICK_SIZE 10000

/* This is always executed with interrupts disabled.  */
unsigned long amiga_gettimeoffset (void)
{
    unsigned short hi, lo, hi2;
    unsigned long ticks, offset = 0;

    /* read CIA B timer A current value */
    hi	= ciab.tahi;
    lo	= ciab.talo;
    hi2 = ciab.tahi;

    if (hi != hi2)
	lo = ciab.talo;

    ticks = hi << 8 | lo;

    if (boot_info.bi_amiga.eclock == NTSC_ECLOCK) {
	if (ticks > NTSC_JIFFY_TICKS - NTSC_JIFFY_TICKS / 100)
	  /* check for pending interrupt */
	  if (ciab.icr & CIA_ICR_TA)
	    offset = 10000;
	ticks = (NTSC_JIFFY_TICKS-1) - ticks;
	ticks = 10000 * ticks / NTSC_JIFFY_TICKS;
    } else {
	if (ticks > PAL_JIFFY_TICKS - PAL_JIFFY_TICKS / 100)
	  /* check for pending interrupt */
	  if (ciab.icr & CIA_ICR_TA)
	    offset = 10000;
	ticks = (PAL_JIFFY_TICKS-1) - ticks;
	ticks = 10000 * ticks / NTSC_JIFFY_TICKS;
    }

    return ticks + offset;
}

void a3000_gettod (struct mktime *timep)
{
	volatile struct tod3000 *tod = TOD_3000;

	tod->cntrl1 = TOD3000_CNTRL1_HOLD;

	timep->sec  = tod->second1 * 10 + tod->second2;
	timep->min  = tod->minute1 * 10 + tod->minute2;
	timep->hour = tod->hour1   * 10 + tod->hour2;
	timep->day  = tod->day1    * 10 + tod->day2;
	timep->mon  = tod->month1  * 10 + tod->month2 - 1;
	timep->year = tod->year1   * 10 + tod->year2;

	tod->cntrl1 = TOD3000_CNTRL1_FREE;
}

void a2000_gettod (struct mktime *timep)
{
	volatile struct tod2000 *tod = TOD_2000;

	tod->cntrl1 = TOD2000_CNTRL1_HOLD;

	while (tod->cntrl1 & TOD2000_CNTRL1_BUSY)
		;

	timep->sec  = tod->second1     * 10 + tod->second2;
	timep->min  = tod->minute1     * 10 + tod->minute2;
	timep->hour = (tod->hour1 & 3) * 10 + tod->hour2;
	timep->day  = tod->day1        * 10 + tod->day2;
	timep->mon  = tod->month1      * 10 + tod->month2 - 1;
	timep->year = tod->year1       * 10 + tod->year2;

	if (!(tod->cntrl3 & TOD2000_CNTRL3_24HMODE))
		if ((!tod->hour1 & TOD2000_HOUR1_PM) && timep->hour == 12)
			timep->hour = 0;
		else if ((tod->hour1 & TOD2000_HOUR1_PM) && timep->hour != 12)
			timep->hour += 12;

	tod->cntrl1 &= ~TOD2000_CNTRL1_HOLD;
}

void amiga_waitbut (void)
{
    int i;

    while (1) {
	while (ciaa.pra & 0x40);

	/* debounce */
	for (i = 0; i < 1000; i++);

	if (!(ciaa.pra & 0x40))
	    break;
    }

    /* wait for button up */
    while (1) {
	while (!(ciaa.pra & 0x40));

	/* debounce */
	for (i = 0; i < 1000; i++);

	if (ciaa.pra & 0x40)
	    break;
    }
}

void ami_serial_print (const char *str)
{
	ushort c;

	while (*str) {
		c = *str++;

		custom.serdat = c | 0x100;
		while (!(custom.serdatr & 0x2000))
			;
		if (c == '\n') {
			custom.serdat = '\r' | 0x100;
			while (!(custom.serdatr & 0x2000))
				;
		}
	}
}

void dbprintf(const char *fmt , ...)
{
	static char buf[1024];
	va_list args;
	extern void console_print (const char *str);
	extern int vsprintf(char * buf, const char * fmt, va_list args);

	va_start(args, fmt);
	vsprintf(buf, fmt, args);
	va_end(args);

	console_print (buf);
}

NORET_TYPE void amiga_reset( void )
    ATTRIB_NORET;

void amiga_reset (void)
{
    unsigned long jmp_addr = VTOP(&&jmp_addr_label);

    cli();
    if (boot_info.cputype & CPU_68040)
	__asm__ __volatile__
	    ("moveq #0,d0\n\t"
	     "bra   2f\n\t"
	     ".align 4\n\t"		/* align on cache line start */
	     "2:\n\t"
	     ".long 0x4e7b0003\n\t"	/* movec d0,tc */
	     "jmp   %0@\n\t"
	     : /* no outputs */
	     : "a" (jmp_addr)
	     : "d0");
    else
	__asm__ __volatile__
	    ("pmove  tc,sp@\n\t"
	     "bclr   #7,sp@\n\t"
	     "pmove  sp@,tc\n\t"
	     "jmp    %0@\n\t"
	     : /* no outputs */
	     : "a" (jmp_addr));
  jmp_addr_label:
    __asm__ __volatile__
	("leal  0x01000000,a0\n\t"
	 "subl  a0@(-0x14),a0\n\t"
	 "movel a0@(4),a0\n\t"
	 "subql #2,a0\n\t"
	 "bra   1f\n\t"
	 /* align on a longword boundary */
	 ".align 2\n\t"
	 "1:\n\t"
	 "reset\n\t"
	 "jmp   a0@");

    for (;;);
}
