/*
 *  linux/init/config.c
 *
 *  Copyright (C) 1993, Hamish Macdonald
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file README.legal in the main directory of this archive
 * for more details.
 */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/bootinfo.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/tty.h>
#include <linux/console.h>

#ifdef CONFIG_AMIGA
#include <linux/amigaconf.h>
#endif
#ifdef CONFIG_ATARI
#include <linux/atariconf.h>
#endif
#ifdef CONFIG_MAC
#include <linux/macconf.h>
#endif

/* setup some dummy routines */
static void dummy_floppy_init (void) {}
static int dummy_floppy_change (struct buffer_head *bh)
{
    return 0;
}
static unsigned long dummy_hd_init (unsigned long start,
				    unsigned long end)
{
    return start;
}
static void dummy_waitbut(void)
{
}
static void dummy_hd_setup (char *str, int *ints)
{
}

void (*mach_sched_init) (isrfunc);
unsigned long (*mach_keyb_init) (unsigned long);
void (*mach_init_INTS) (void);
int (*mach_add_isr) (unsigned long, isrfunc, int, void *);
unsigned long (*mach_gettimeoffset) (void);
void (*mach_gettod) (struct mktime*);
void (*mach_check_partition) (struct gendisk *, unsigned int);
void (*mach_mksound)( unsigned int count, unsigned int ticks );
void (*mach_reset)( void );
void (*mach_floppy_init) (void) = dummy_floppy_init;
int (*mach_floppy_change) (struct buffer_head *bh) = dummy_floppy_change;
unsigned long (*mach_hd_init) (unsigned long, unsigned long) = dummy_hd_init;
void (*waitbut)(void) = dummy_waitbut;
void (*mach_hd_setup) (char *, int *) = dummy_hd_setup;

void config_init(void)
{
	switch (boot_info.machtype) {
#ifdef CONFIG_AMIGA
	    case MACH_AMIGA:
		config_amiga();
		break;
#endif
#ifdef CONFIG_ATARI
	    case MACH_ATARI:
		config_atari();
		break;
#endif
#ifdef CONFIG_MAC
	    case MACH_MAC:
		config_mac();
		break;
#endif
	    default:
		panic ("No configuration setup");
	}
}
